 
// Пин подключен к ST_CP входу 74HC595
int latchPin = 8, voltageX10,voltageGuard,ons,dz,temp,tempHop,
// Пин подключен к SH_CP входу 74HC595
clockPin = 2, //12
// Пин подключен к DS входу 74HC595
dataPin = 3; //11

unsigned long voltageTime,powerDownBurnTimeLast = 0,powerTime,ct;

// Коэффициент вычисления напряжения (на него делится)
float voltageX10Coefficient = 5.12;

// Сдвиг вычисления напряжения (он прибавляется)
int voltageX10Shift = 35;

// Выход на управление релюхой питания
byte relay220Pin = 7;

// Выход на управление вентилятором
byte fanPin = 6; //На схеме - 9

// Уровень мощости (начальный)
byte powerLevel = 0;

// Максимальное напряжение * 10
byte maxVoltageX10 = 148;

// Напряжение * 10, при котором уже нельзя увеличивать мощность
byte powerUpMaxTreshold = 142;

// Напряжение * 10, при котором мощность повысится принудительно
byte powerUpMinTreshold = 136;

// Если между повышением мощности по powerUpMinTreshold и её понижением прошло менее чем столько миллисекунд, то powerUpMinTreshold снизится на 1
unsigned int powerUpMinTresholdTimeout = 60000;

// Аналоговый вход для мониторинга напряжения
byte voltageInput = 5;

// Аналоговый вход для мониторинга количества включённых релюх
byte guardInput = 4;

// Аналоговый вход для терморезистора
byte tempInput = 1;

// Минимально допустимое значение на входе для терморезистора
byte maxTemp = 150;

// Значение на входе для терморезистора, меньше которого уже нельзя повышать мощность
byte maxTempUp = 200;

// Переменная, показывающая текущий статус защиты от перегрева
boolean isBurning = false;

// Частота обновления уровня напряжения на экране, в миллисекундах
unsigned int voltageRefreshTime = 180;

// Задержка в миллисекундах перед включением зарядки
unsigned int powerOnDelay = 2000;

// Время, через которое можно повышать мощность на один уровень, в миллисекундах
unsigned int powerUpTime = 65000;

// Время, через которое можно понижать мощность по перегреву на один уровень, в миллисекундах
unsigned int powerDownBurnTime = 65000;

//Пауза после понижения мощности, в миллисекундах
int powerDownPause = 500;

//Количество миллисекундных итераций проверки напряжения перед поднятием мощности
int upCycles = 5000;

// Максимально допустимое значение guardInput
byte guardTreshold = 160;

// Длительность перехода между релюхами в миллисекундах
byte relayTransition = 80;

// Шаблоны для экрана
char* dozens[]={"11011111","11001111","11101111","11101011","11111011","10111011","10111111","10111101","11111101","11110101","11110111","11010111"};
char* dozensBurn[]={"11011110","11001110","11101110","11101010","11111010","10111010","10111110","10111100","11111100","11110100","11110110","11010110"};

// Шаблоны для блока реле
char* relayp[]={"00000000","10000001","00000011","00000110","00001100","00011000","00110000","01100000","00000000"};
char* relays[]={"00000000","10000000","00000001","00000010","00000100","00001000","00010000","00100000","01000000"};

// Настройки скорости вращения вентилятора
byte fanSpeeds[]={0,0,80,90,100,110,120,180,255};

boolean goingUp = true, isUpAfter = false; 
 
void setup() {
  pinMode(latchPin, OUTPUT);
  pinMode(clockPin, OUTPUT);
  pinMode(dataPin, OUTPUT);
  pinMode(relay220Pin, OUTPUT);
  pinMode(fanPin, OUTPUT);
  digitalWrite(relay220Pin, LOW);
  digitalWrite(relay220Pin, LOW);
}
 
void loop() {
  ct=millis();
  voltageX10=(analogRead(voltageInput)+voltageX10Shift)/voltageX10Coefficient;
  voltageGuard=analogRead(guardInput);
  temp=analogRead(tempInput);
  if (temp > maxTemp - tempHop) {isBurning = false;} else {isBurning = true;}
  if (ct < 1000) {digitalWrite(relay220Pin, LOW);}
  if (temp < maxTempUp) {analogWrite(fanPin,255);}

// Проверка на количество включенных релюх и автоматическое отключение - восстановление - включение
  if (voltageGuard > guardTreshold) {
    digitalWrite(relay220Pin, LOW);
    while (voltageGuard > guardTreshold) {
    digitalWrite(latchPin, LOW);
    if (isBurning == false) {shiftOut(dataPin, clockPin, MSBFIRST, bitseq(dozens[(millis() % 12)]));} else {shiftOut(dataPin, clockPin, MSBFIRST, bitseq(dozensBurn[(millis() % 12)]));} 
    shiftOut(dataPin, clockPin, MSBFIRST, bitseq(relays[powerLevel])); 
    digitalWrite(latchPin, HIGH);
    voltageGuard=analogRead(guardInput);  
    }
    digitalWrite(relay220Pin, HIGH); 
  }

// Обновление информации на экранчике
  if (ct > voltageTime + voltageRefreshTime) {
    ons = voltageX10 % 10;
    dz = voltageX10 / 10;
    digitalWrite(latchPin, LOW);
    if (isBurning == false) {shiftOut(dataPin, clockPin, MSBFIRST, bitseq(dozens[(millis() % 12)]));} else {shiftOut(dataPin, clockPin, MSBFIRST, bitseq(dozensBurn[(millis() % 12)]));} 
    shiftOut(dataPin, clockPin, MSBFIRST, bitseq(relays[powerLevel])); 
    digitalWrite(latchPin, HIGH);
    voltageTime = ct;
  }

// Повышение мощности
  if ((temp > maxTempUp) && (powerLevel < 8) && (ct > powerTime + powerUpTime) && ((goingUp == true && voltageX10 < powerUpMaxTreshold) || (goingUp == false && voltageX10 < powerUpMinTreshold))) {
    boolean stillLow = true;
    if (goingUp == false) {
      for (int x=0; x <= upCycles; x++){
         voltageX10=(analogRead(voltageInput)+voltageX10Shift)/voltageX10Coefficient;
         if (voltageX10 > powerUpMinTreshold) {stillLow=false; break;}
         delay(1);
      }
    }
    if (stillLow == true) {
      tempHop = 0;
      powerLevel=powerLevel+1;
      digitalWrite(latchPin, LOW);
      if (isBurning == false) {shiftOut(dataPin, clockPin, MSBFIRST, bitseq(dozens[(millis() % 12)]));} else {shiftOut(dataPin, clockPin, MSBFIRST, bitseq(dozensBurn[(millis() % 12)]));} 
      shiftOut(dataPin, clockPin, MSBFIRST, bitseq(relayp[powerLevel-1])); 
      digitalWrite(latchPin, HIGH);
      delay(relayTransition);
      digitalWrite(latchPin, LOW);
      if (isBurning == false) {shiftOut(dataPin, clockPin, MSBFIRST, bitseq(dozens[(millis() % 12)]));} else {shiftOut(dataPin, clockPin, MSBFIRST, bitseq(dozensBurn[(millis() % 12)]));} 
      shiftOut(dataPin, clockPin, MSBFIRST, bitseq(relays[powerLevel])); 
      digitalWrite(latchPin, HIGH);
      if (temp < maxTempUp) {analogWrite(fanPin,255);} else {analogWrite(fanPin,fanSpeeds[powerLevel]);}
      if (powerLevel == 1) {
        delay(powerOnDelay);
        digitalWrite(relay220Pin, HIGH);
      }
      powerTime = ct;
      if (goingUp == false) {isUpAfter = true;}
    }
  }

// Понижение мощности
  if ((voltageX10 > maxVoltageX10) || ((isBurning == true) && (ct > powerDownBurnTimeLast + powerDownBurnTime))) {
    if (powerLevel > 1) {
      if (isBurning == true) {powerDownBurnTimeLast = ct; tempHop=tempHop + 5;}
      powerLevel=powerLevel-1;
      voltageX10=(analogRead(voltageInput)+voltageX10Shift)/voltageX10Coefficient;
      digitalWrite(latchPin, LOW);
      if (isBurning == false) {shiftOut(dataPin, clockPin, MSBFIRST, bitseq(dozens[(millis() % 12)]));} else {shiftOut(dataPin, clockPin, MSBFIRST, bitseq(dozensBurn[(millis() % 12)]));} 
      shiftOut(dataPin, clockPin, MSBFIRST, bitseq(relayp[powerLevel])); 
      digitalWrite(latchPin, HIGH);
      delay(relayTransition);
      digitalWrite(latchPin, LOW);
      if (isBurning == false) {shiftOut(dataPin, clockPin, MSBFIRST, bitseq(dozens[(millis() % 12)]));} else {shiftOut(dataPin, clockPin, MSBFIRST, bitseq(dozensBurn[(millis() % 12)]));} 
      shiftOut(dataPin, clockPin, MSBFIRST, bitseq(relays[powerLevel])); 
      digitalWrite(latchPin, HIGH);
      if ((isUpAfter == true) && (powerTime > ct - powerUpMinTresholdTimeout) && (temp > maxTemp)) {powerUpMinTreshold = powerUpMinTreshold - 1;}
      delay(powerDownPause);
      goingUp = false;     
      analogWrite(fanPin,fanSpeeds[powerLevel]);
   } else {
// Конец зарядки, тушите свет, мигайте экраном!
      digitalWrite(relay220Pin, LOW);
      analogWrite(fanPin,fanSpeeds[0]);
      while (goingUp == false) {
        digitalWrite(latchPin, LOW);
        shiftOut(dataPin, clockPin, MSBFIRST, bitseq("11111111")); 
        shiftOut(dataPin, clockPin, MSBFIRST, bitseq("00000000")); 
        digitalWrite(latchPin, HIGH);
        delay(500);
        voltageX10=(analogRead(voltageInput)+voltageX10Shift)/voltageX10Coefficient;
        ons = voltageX10 % 10;
        dz = voltageX10 / 10;
        digitalWrite(latchPin, LOW);
        shiftOut(dataPin, clockPin, MSBFIRST, bitseq(dozens[(millis() % 12)])); 
        shiftOut(dataPin, clockPin, MSBFIRST, bitseq("00000000")); 
        digitalWrite(latchPin, HIGH);
        delay(500);
      }
    }
  }
}

byte bitseq (char s[8]){
byte ab=0;
for (byte ba=0; ba<8; ba++){
  if (s[ba] == '0') {bitWrite(ab,ba,0);} else {bitWrite(ab,ba,1);}  
}
return ab;
}
